@@ -13,7 +13,7 @@ from xml.dom.minidom import parseString, parse
 from collections import Mapping
 
 #TODO rewrite this to simple import
-from art3dutils.utilities import (
+from .utilities import (
     get_polygon_nodes, process_node, get_outline_container,
     get_text_nodes, get_svg_viewbox,
     get_text_coords, remove_child_nodes,
@@ -22,17 +22,18 @@ from art3dutils.utilities import (
 )
 
 ATTR_TYPES = {
-    'quarter_number': 'int',
-    'building_number': 'int',
-    'section_number': 'int',
-    'floor_number': 'int',
-    'number': 'int',
-    'room_count': 'int',
-    'square': 'float',
-    'status': 'int',
-    'cost_per_meter': 'int',
-    'type': 'unicode',
-    'note': 'unicode'
+    'quarter_number': ('q', 'int'),
+    'building_number': ('b', 'int'),
+    'section_number': ('s', 'int'),
+    'floor_number': ('f', 'int'),
+    'number': ('n', 'int'),
+    'room_count': ('rc', 'int'),
+    'square': ('sq', 'float'),
+    'status': ('st', 'int'),
+    'total_cost': ('tc', 'int'),
+    'cost_per_meter': ('cpm', 'int'),
+    'type': ('t', 'unicode'),
+    'note': ('nt', 'unicode')
 }
 
 session = None
@@ -71,9 +72,9 @@ class ApartmentError(Exception):
 class FloorError(Exception):
 
     def __init__(self, floor):
-        message ='No svg path for floor: q%d-b%d-s%d-f%d' % \
-                 (floor.quarter_number, floor.building_number,
-                  floor.section_number, floor.number)
+        message = 'No svg path for floor: q%d-b%d-s%d-f%d' % \
+                  (floor.quarter_number, floor.building_number,
+                   floor.section_number, floor.number)
 
         Exception.__init__(self, message)
         self.floor = floor
@@ -81,6 +82,11 @@ class FloorError(Exception):
 
 class Entity(object):
 
+    @property
+    def fs_name(self):
+        """Return class name for use in filesystem (directories, etc.)"""
+        return self.__class__.__name__.lower()
+
     @classmethod
     def fetch(cls, key):
         return session.query(cls).get(key)
@@ -120,6 +126,13 @@ class Project(Entity):
         self.title = title
         self.quarters = quarters or []
 
+    def fetch_apartments(self, available_only=False):
+        query = session.query(Apartment)\
+                       .filter(apartments.c.project_title == self.title)
+        if available_only:
+            query = query.filter(apartments.c.status == 1)
+        return query.all()
+
 
 class Quarter(Entity):
 
@@ -130,24 +143,34 @@ class Quarter(Entity):
     def fetch_available_buildings(self):
         available = []
         for building in self.buildings:
-            if len(building.get_apartments(True)) > 0:
+            if len(building.fetch_apartments(True)) > 0:
                 available.append(building)
         return available
 
 
-class Building(Entity):
+class Building(Entity, Mapping):
 
     def __init__(self, number=0, sections=None):
         self.number = number
         self.sections = sections or []
 
-    def get_floors(self, floor_numbers=None):
+    def __len__(self):
+        return len(self.__dict__)
+
+    def __iter__(self):
+        for key, value in self.__dict__.items():
+            yield key
+
+    def __getitem__(self, key):
+        return self.__dict__.get(key)
+
+    def fetch_floors(self, floor_numbers=None):
         query = session.query(Floor).filter(floors.c.building_number==self.number)
         if floor_numbers:
             query = query.filter(floors.c.number.in_(list(floor_numbers)))
         return query.all()
 
-    def get_apartments(self, available_only=False, floors=None):
+    def fetch_apartments(self, available_only=False, floors=None):
         query = session.query(Apartment)\
                        .filter(apartments.c.project_title==self.project_title,
                                apartments.c.quarter_number==self.quarter_number,
@@ -159,10 +182,6 @@ class Building(Entity):
             query = query.filter(apartments.c.status == 1)
         return query.all()
 
-    @property
-    def apartments(self):
-        return self.get_apartments()
-
     def get_sections(self, reverse=False):
         """Get sections sorted"""
         sections = self.sections
@@ -186,20 +205,19 @@ class Section(Entity, Mapping):
     def __getitem__(self, key):
         return self.__dict__.get(key)
 
-    def get_apartments(self, available_only=False, room_count=None):
+    def fetch_apartments(self, available_only=False, room_count=None):
         query = session.query(Apartment)\
                        .filter(apartments.c.project_title==self.project_title,
                                apartments.c.quarter_number==self.quarter_number,
                                apartments.c.building_number==self.building_number,
                                apartments.c.section_number==self.number)
-        apts = query.all()
         if available_only:
-            apts = filter(lambda apt: apt.status is 1, apts)
-        if room_count is not None:
-            apts = filter(lambda apt: apt.room_count is room_count, apts)
-        return apts
+            query = query.filter(apartments.c.status == 1)
+        if room_count:
+            query = query.filter(apartments.c.room_count == room_count)
+        return query.all()
 
-    def get_floors(self, available_only=False):
+    def fetch_floors(self, available_only=False):
         all_floors = session.query(Floor)\
                        .filter(apartments.c.project_title==self.project_title,
                                floors.c.quarter_number==self.quarter_number,
@@ -208,7 +226,7 @@ class Section(Entity, Mapping):
         if available_only:
             available_floors = []
             for floor in all_floors:
-                if len(floor.get_apartments(available_only=True)):
+                if len(floor.fetch_apartments(available_only=True)):
                     available_floors.append(floor)
             return available_floors
         else:
@@ -277,6 +295,11 @@ class Floor(Entity, Mapping):
         result = sorted(apts, key=lambda apt: apt.number, reverse=reverse)
         return result
 
+    def fetch_apartments(self, available_only=False, reverse=False,
+                         room_count=None):
+        """Mock fetch_apartments as uniform method"""
+        return self.get_apartments(available_only, reverse, room_count)
+
     def pick_file(self, dir_):
         """Return appropriate file path"""
         return self.apartments[0].pick_file(dir_)
@@ -416,8 +439,10 @@ class Apartment(Entity, Mapping):
         else:
             outline_node = self._get_node()
         if not outline_node:
-            raise ApartmentError(self, message_point='No outline in %s' %
-                                                     orig_svg_path)
+            raise ApartmentError(self,
+                                 message_point='No outline in %s '
+                                               '(out of total %d outlines)' %
+                                               (orig_svg_path, len(nodes)))
         return area_coords(outline_node)
 
     def get_outline_id(self, floor_dir, complex=False):
@@ -425,7 +450,7 @@ class Apartment(Entity, Mapping):
         outline_node = None
         svg_path = self.pick_file(floor_dir)
         nodes = get_polygon_nodes(svg_path)
-        apartments = self.floor.get_apartments(reverse=True)
+        apartments = self.floor.fetch_apartments(reverse=True)
         for (apartment, node) in zip(apartments, nodes):
             if apartment is self:
                 outline_node = node
@@ -583,7 +608,7 @@ class Apartment(Entity, Mapping):
         floor_dom = parse(self.pick_file(path, True))
 
         if text_fill:
-            apartments = self.floor.get_apartments(reverse=True)
+            apartments = self.floor.fetch_apartments(reverse=True)
             for (apartment, text_node) in zip(apartments,
                                               get_text_nodes(dom_=floor_dom)):
                 remove_child_nodes(text_node)
@@ -689,104 +714,141 @@ class Apartment(Entity, Mapping):
 
 
 projects = Table('project', metadata,
-    Column('title', VARCHAR(255), primary_key=True, nullable=False),
-    mysql_engine='MyISAM', mysql_charset='utf8')
+                 Column('title', VARCHAR(255), primary_key=True,
+                        nullable=False),
+                 mysql_engine='MyISAM', mysql_charset='utf8')
 
 quarters = Table('quarter', metadata,
-    Column('project_title', VARCHAR(255), ForeignKey('project.title'),
-            primary_key=True),
-    Column('number', Integer, primary_key=True, nullable=False,
-            autoincrement=False),
-    mysql_engine='MyISAM', mysql_charset='utf8')
+                 Column('project_title', VARCHAR(255),
+                        ForeignKey('project.title'),
+                        primary_key=True),
+                 Column('number', Integer, primary_key=True, nullable=False,
+                        autoincrement=False),
+                 mysql_engine='MyISAM', mysql_charset='utf8')
 
 buildings = Table('building', metadata,
-    Column('project_title', VARCHAR(255), ForeignKey('quarter.project_title'),
-            primary_key=True),
-    Column('quarter_number', Integer, ForeignKey('quarter.number'),
-            primary_key=True),
-    Column('number', Integer, primary_key=True, nullable=False,
-            autoincrement=False), mysql_engine='MyISAM', mysql_charset='utf8'
-    )
+                  Column('project_title', VARCHAR(255),
+                         ForeignKey('quarter.project_title'),
+                         primary_key=True),
+                  Column('quarter_number', Integer,
+                         ForeignKey('quarter.number'),
+                         primary_key=True),
+                  Column('number', Integer, primary_key=True, nullable=False,
+                         autoincrement=False), mysql_engine='MyISAM',
+                  mysql_charset='utf8')
 
 sections = Table('section', metadata,
-    Column('project_title', VARCHAR(255), ForeignKey('building.project_title'),
-            primary_key=True),
-    Column('quarter_number', Integer, ForeignKey('building.quarter_number'),
-            primary_key=True),
-    Column('building_number', Integer, ForeignKey('building.number'),
-            primary_key=True),
-    Column('number', Integer, primary_key=True, nullable=False,
-            autoincrement=False), mysql_engine='MyISAM', mysql_charset='utf8'
-    )
+                 Column('project_title', VARCHAR(255),
+                        ForeignKey('building.project_title'),
+                        primary_key=True),
+                 Column('quarter_number', Integer,
+                        ForeignKey('building.quarter_number'),
+                        primary_key=True),
+                 Column('building_number', Integer,
+                        ForeignKey('building.number'),
+                        primary_key=True),
+                 Column('number', Integer, primary_key=True, nullable=False,
+                        autoincrement=False), mysql_engine='MyISAM',
+                 mysql_charset='utf8')
 
 floors = Table('floor', metadata,
-    Column('project_title', VARCHAR(255), ForeignKey('section.project_title'),
-            primary_key=True),
-    Column('quarter_number', Integer, ForeignKey('section.quarter_number'),
-            primary_key=True),
-    Column('building_number', Integer, ForeignKey('section.building_number'),
-            primary_key=True),
-    Column('section_number', Integer, ForeignKey('section.number'),
-            primary_key=True),
-    Column('number', Integer, primary_key=True, nullable=False,
-            autoincrement=False),
-    mysql_engine='MyISAM', mysql_charset='utf8')
+               Column('project_title', VARCHAR(255),
+                      ForeignKey('section.project_title'),
+                      primary_key=True),
+               Column('quarter_number', Integer,
+                      ForeignKey('section.quarter_number'),
+                      primary_key=True),
+               Column('building_number', Integer,
+                      ForeignKey('section.building_number'),
+                      primary_key=True),
+               Column('section_number', Integer, ForeignKey('section.number'),
+                      primary_key=True),
+               Column('number', Integer, primary_key=True, nullable=False,
+                      autoincrement=False),
+               mysql_engine='MyISAM', mysql_charset='utf8')
 
 apartments = Table('apartment', metadata,
-    Column('project_title', VARCHAR(255), ForeignKey('floor.project_title'),
-            primary_key=True),
-    Column('quarter_number', Integer, ForeignKey('floor.quarter_number'),
-            primary_key=True),
-    Column('building_number', Integer, ForeignKey('floor.building_number'),
-            primary_key=True),
-    Column('section_number', Integer, ForeignKey('floor.section_number'),
-            primary_key=True),
-    Column('floor_number', Integer, ForeignKey('floor.number'),
-            primary_key=True),
-    Column('number', Integer, primary_key=True, nullable=False,
-            autoincrement=False),
-    Column('room_count', Integer),
-    Column('square', Float),
-    Column('square_out', Float),
-    Column('total_cost', Integer),
-    Column('cost_per_meter', Integer),
-    Column('status', Integer),
-    Column('open', Boolean),
-    Column('note', VARCHAR(255)),
-    Column('type', VARCHAR(255))
-    , mysql_engine='MyISAM', mysql_charset='utf8')
+                   Column('project_title', VARCHAR(255),
+                          ForeignKey('floor.project_title'),
+                          primary_key=True),
+                   Column('quarter_number', Integer,
+                          ForeignKey('floor.quarter_number'),
+                          primary_key=True),
+                   Column('building_number', Integer,
+                          ForeignKey('floor.building_number'),
+                          primary_key=True),
+                   Column('section_number', Integer,
+                          ForeignKey('floor.section_number'),
+                          primary_key=True),
+                   Column('floor_number', Integer, ForeignKey('floor.number'),
+                          primary_key=True),
+                   Column('number', Integer, primary_key=True, nullable=False,
+                          autoincrement=False),
+                   Column('room_count', Integer),
+                   Column('square', Float),
+                   Column('square_out', Float),
+                   Column('total_cost', Integer),
+                   Column('cost_per_meter', Integer),
+                   Column('status', Integer),
+                   Column('open', Boolean),
+                   Column('note', VARCHAR(255)),
+                   Column('type', VARCHAR(255)),
+                   mysql_engine='MyISAM',
+                   mysql_charset='utf8')
 
 mapper(Project, projects, properties={
-    'quarters': relationship(Quarter, backref='project')
+    'quarters': relationship(Quarter, backref='project',
+                             cascade="all, delete-orphan")
 })
 mapper(Quarter, quarters, properties={
     'buildings': relationship(Building, backref='quarter',
-        primaryjoin=and_(
-            buildings.c.quarter_number==quarters.c.number,
-            buildings.c.project_title==quarters.c.project_title
-        ))})
+                              primaryjoin=and_(
+                                  buildings.c.quarter_number ==
+                                  quarters.c.number,
+                                  buildings.c.project_title ==
+                                  quarters.c.project_title
+                              ),
+                              cascade="all, delete-orphan")
+})
 mapper(Building, buildings, properties={
     'sections': relationship(Section, backref='building',
-        primaryjoin=and_(
-            sections.c.building_number==buildings.c.number,
-            sections.c.quarter_number==buildings.c.quarter_number,
-            sections.c.project_title==buildings.c.project_title
-        ))})
+                             primaryjoin=and_(
+                                 sections.c.building_number ==
+                                 buildings.c.number,
+                                 sections.c.quarter_number ==
+                                 buildings.c.quarter_number,
+                                 sections.c.project_title ==
+                                 buildings.c.project_title
+                             ),
+                             cascade="all, delete-orphan")
+})
 mapper(Section, sections, properties={
     'floors': relationship(Floor, backref='section',
-        primaryjoin=and_(
-            floors.c.section_number==sections.c.number,
-            floors.c.building_number==sections.c.building_number,
-            floors.c.quarter_number==sections.c.quarter_number,
-            floors.c.project_title==sections.c.project_title
-        ))})
+                           primaryjoin=and_(
+                               floors.c.section_number == sections.c.number,
+                               floors.c.building_number ==
+                               sections.c.building_number,
+                               floors.c.quarter_number ==
+                               sections.c.quarter_number,
+                               floors.c.project_title ==
+                               sections.c.project_title
+                           ),
+                           cascade="all, delete-orphan")
+})
 mapper(Floor, floors, properties={
     'apartments': relationship(Apartment, backref='floor',
-        primaryjoin=and_(
-            apartments.c.floor_number==floors.c.number,
-            apartments.c.section_number==floors.c.section_number,
-            apartments.c.building_number==floors.c.building_number,
-            apartments.c.quarter_number==floors.c.quarter_number,
-            apartments.c.project_title==floors.c.project_title
-        ))})
+                               primaryjoin=and_(
+                                   apartments.c.floor_number ==
+                                   floors.c.number,
+                                   apartments.c.section_number ==
+                                   floors.c.section_number,
+                                   apartments.c.building_number ==
+                                   floors.c.building_number,
+                                   apartments.c.quarter_number ==
+                                   floors.c.quarter_number,
+                                   apartments.c.project_title ==
+                                   floors.c.project_title
+                               ),
+                               cascade="all, delete-orphan")
+})
 mapper(Apartment, apartments)
\ No newline at end of file
